#version 130
#extension GL_EXT_gpu_shader4 : enable

uniform sampler2DArray texture;
uniform vec3 focal;
uniform vec3 lightVector;
in vec4 texCoord;
in vec4 colordata;
in vec3 viewVector;
in float worldLight;
in float vertdist;

const float FOG_DENSITY   = 0.5;
const float FOG_LUMINANCE = 0.5;
const float FOG_MAXIMUM   = 0.7;

const float ALPHA_EXP  = 0.4;

const float SHADOW_EXP = 1.0;
const float SHADOW_MUL = 1.2;
const float SHADOW_MAX = 0.98;

#ifdef FOG_LUM
vec4 hjemmebakt_fog(vec4 color){
	float f = min(FOG_MAXIMUM, pow(vertdist / focal[2] * 1.05, 1.25) * FOG_DENSITY);
	float lum = min(1.0, (0.7 + 0.2126 * color.r + 0.7152 * color.g + 0.0722 * color.b) * FOG_LUMINANCE);
	return vec4(mix(color.rgb, vec3(lum), f), color[3]);
}
#endif

void main(void) {

	if (vertdist > focal[2]) discard; //ze end of ze world!
	vec4 color = texture2DArray(texture, texCoord.xyz);
	
	if (color.a == 0) discard; //le discard!
	
	color.rgb *= worldLight;
	// upramp shadow gamma
	float alpha = min(SHADOW_MAX,
					smoothstep(0.0, 1.0, pow(colordata[3], 0.9) )
					);
	vec4 light = mix(color, colordata, alpha);
	// light[3] = color[3];
	
	float fadedist = focal[2] * 0.88;
	if (vertdist > fadedist) {
		light[3] = max(0.0, 1.0 - (vertdist - fadedist) / (focal[2] - fadedist) );
	} else {
		light[3] = 1.0; //opaque
	}
	
#ifdef FOG_LUM
	gl_FragColor = hjemmebakt_fog(light);
#else
	gl_FragColor = light;
#endif
}
